# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.jc_api_v1.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is a utility library for the jumpcloud version 1 api

.. note::

    To learn more about the jumpcloud api 1
    `project website <https://github.com/TheJumpCloud/jcapi-python/tree/master/jcapiv1>`_.
"""
from distutils.util import strtobool

import jcapiv1
from jcapiv1 import Systemuserput
from jcapiv1.rest import ApiException
from jccli.errors import SystemUserNotFoundError

from jccli.helpers import class_to_dict

# pylint: disable=too-many-arguments
class JumpcloudApiV1:
    """
        Wrapper for Jumpcloud API v1
    """
    def __init__(self, api_key):
        configuration = jcapiv1.Configuration()
        configuration.api_key['x-api-key'] = api_key
        self.system_users_api = jcapiv1.SystemusersApi(jcapiv1.ApiClient(configuration))

    def get_users(self, limit='100', skip=0, search='', filter='', sort='', fields=''):
        """
        Get users from jumpcloud
        :param limit:
        :param skip:
        :param search:
        :param filter:
        :param sort:
        :param fields:
        :return: a list of users with dict of settings
        """
        try:
            api_response = self.system_users_api.systemusers_list(content_type='application/json',
                                                                  accept='application/json',
                                                                  limit=limit,
                                                                  skip=skip,
                                                                  sort=sort,
                                                                  fields=fields,
                                                                  x_org_id='',
                                                                  search=search,
                                                                  filter=filter)
            users = class_to_dict(api_response.results)
            return users
        except ApiException as error:
            raise "Exception when calling SystemusersApi->systemusers_list: %s\n" % error

    def create_user(self, systemuser):
        """
        Create a new user in jumpcloud
        :param systemuser: a dictoionary of Systemuser properties
               https://github.com/TheJumpCloud/jcapi-java/blob/master/jcapiv1/docs/Systemuser.md
        :return: The api response
        """
        body = jcapiv1.Systemuserputpost(username=systemuser['username'],
                                         email=systemuser['email'],
                                         firstname=systemuser.get('firstname', ''),
                                         lastname=systemuser.get('lastname', ''),
                                         allow_public_key=strtobool(
                                             systemuser.get('allow_public_key', 'True')),
                                         ldap_binding_user=strtobool(
                                             systemuser.get('ldap_binding_user', 'False')),
                                         passwordless_sudo=strtobool(
                                             systemuser.get('passwordless_sudo', 'False')),
                                         sudo=strtobool(systemuser.get('sudo', 'False')))
        try:
            api_response = self.system_users_api.systemusers_post(content_type='application/json',
                                                                  accept='application/json',
                                                                  body=body,
                                                                  x_org_id='')
            return api_response.to_dict()
        except ApiException as error:
            # FIXME: What should this behavior actually be?
            raise Exception("Exception when calling SystemusersApi->systemusers_post: %s\n" % error)

    def delete_user(self, username):
        """
        Delete a user from jumpcloud
        :param id: The jumpcloud id of the user
        :return:
        """
        user_id = self.get_user_id(username)
        if user_id is None:
            raise SystemUserNotFoundError(f"System user {username} not found")

        try:
            api_response = self.system_users_api.systemusers_delete(user_id,
                                                                    content_type='application/json',
                                                                    accept='application/json',
                                                                    x_org_id='')
            return api_response
        except ApiException as error:
            raise "Exception when calling SystemusersApi->systemusers_post: %s\n" % error

    def get_user_id(self, username):
        """
        Get the jumpcloud user id from the user name
        :param username
        :return:  the user id
        """
        users = self.get_users(limit='', fields="username")

        for user in users:
            if user.username == username:
                return user.id

        raise SystemUserNotFoundError('No user found for username: %s' % (username,))

    def get_user(self, username):
        """
        Get detail view of a user object.
        :param user_id:
        :return: user properties dict
        """
        # FIXME: As soon as we figure out how the `filter` parameter works on systemusers_list(), we should start
        #  filtering based on username
        users = self.system_users_api.systemusers_list(
            accept='application/json',
            content_type='application/json'
        ).results

        for user in users:
            if user.username == username:
                return user.to_dict()

        raise SystemUserNotFoundError('No user found for username: %s' % (username,))

    def set_user(self, username, attributes):
        user_id = self.get_user_id(username)
        api_response = self.system_users_api.systemusers_put(
            accept='application/json',
            content_type='application/json',
            id=user_id,
            body=Systemuserput(**attributes)
        )
        return api_response.to_dict()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.cli
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>
"""
import json
import logging
import click
import click_log
import jccli.helpers as jccli_helpers

from jccli.errors import MissingRequiredArgumentError
from jccli.jc_api_v1 import JumpcloudApiV1
from jccli.jc_api_v2 import JumpcloudApiV2
from .__init__ import __version__

# setup click-log
LOGGER = logging.getLogger(__name__)
click_log.basic_config(LOGGER)


def abort_if_false(ctx, param, value):
    # pylint: disable=unused-argument
    """
    A click handler for user prompts
    """
    if not value:
        ctx.abort()


@click.group()
@click.option('--key', "-k", required=True, type=str, help='Jumpcloud API key')
@click_log.simple_verbosity_option(LOGGER)
@click.version_option(version=__version__)
@click.pass_context
def cli(ctx, key):
    """
    Run jccli.
    """
    ctx.obj = {
        'key': key
    }


@cli.group()
@click.pass_context
def user(ctx):
    """
    Command set for users
    """
    pass


@user.command('create')
@click.option('--username', '-u', required=True, type=str)
@click.option('--email', '-e', required=True, type=str)
@click.option('--first-name', '-f', type=str, default='')
@click.option('--last-name', '-l', type=str, default='')
@click.option('--allow-public-key/--disallow-public-key', default=True)
@click.option('--ldap-binding-user', is_flag=True)
@click.option('--passwordless-sudo', is_flag=True)
@click.option('--sudo', is_flag=True)
@click.pass_context
def create_user(ctx, username, email, first_name, last_name, allow_public_key, ldap_binding_user, passwordless_sudo,
                sudo):
    """
    Create a new user
    """
    api1 = JumpcloudApiV1(ctx.obj.get('key'))
    systemuser = {
        'username': username,
        'email': email,
        'first_name': first_name,
        'last_name': last_name,
        'allow_public_key': str(allow_public_key),
        'ldap_binding_user': str(ldap_binding_user),
        'passwordless_sudo': str(passwordless_sudo),
        'sudo': str(sudo)
    }
    response = json.dumps(api1.create_user(systemuser), indent=2)
    LOGGER.info(f"{response}")


@user.command("get")
@click.option('--username', '-u', required=True, type=str)
@click.pass_context
def get_user(ctx, username):
    """
    Detail view of user, outputted in JSON.
    """
    api1 = JumpcloudApiV1(ctx.obj.get('key'))
    response = api1.get_user(username=username)
    serialized_response = json.dumps(response, indent=2)
    LOGGER.info(f"{serialized_response}")


@user.command('set')
@click.option('--username', '-u', required=True, type=str)
@click.option('--email', '-e', default=None, type=str)
@click.option('--firstname', '-f', default=None, type=str)
@click.option('--lastname', '-l', default=None, type=str)
@click.pass_context
def set_user(ctx, username, email, firstname, lastname):
    api1 = JumpcloudApiV1(ctx.obj.get('key'))

    attributes = {}
    if email is not None:
        attributes['email'] = email
    if firstname is not None:
        attributes['firstname'] = firstname
    if lastname is not None:
        attributes['lastname'] = lastname

    response = json.dumps(api1.set_user(username, attributes=attributes), indent=2)
    LOGGER.info(f'{response}')


@user.command("delete")
@click.option('--username', "-u", required=True, type=str)
@click.pass_context
def delete_user(ctx, username):
    """
    Delete a user
    """
    api1 = JumpcloudApiV1(ctx.obj.get('key'))
    response = api1.delete_user(username=username)


@cli.group()
@click.pass_context
def group(ctx):
    """
    Command set for groups
    """
    pass


@group.command('create')
@click.option('--name', "-n", required=True, type=str, help='Name of the group')
@click.option('--type', "-t", required=True, type=click.Choice(['user', 'system']),
              help='The type of group')
@click.pass_context
def create_group(ctx, name, type):
    """
    Create a group
    """
    api2 = JumpcloudApiV2(ctx.obj.get('key'))
    response = api2.create_group(name, type)
    LOGGER.info(f"{response}")


@group.command('delete')
@click.option('--name', "-n", required=True, type=str, help='Name of the group')
@click.pass_context
def delete_group(ctx, name):
    """
    Delete a group
    """
    # FIXME: Ideally, this would output JSON info of the deleted group (similar to delete-user), but at the moment it's
    #  unclear exactly how to do that, given the API wrappers that we have.
    api2 = JumpcloudApiV2(ctx.obj.get('key'))
    api2.delete_group(name)
    LOGGER.info(f"Group {name} deleted")


@cli.command()
@click.option('--data', "-d", required=True, type=click.Path(exists=True),
              help='The JC data file')
@click.option('--yes', "-y", is_flag=True, callback=abort_if_false,
              expose_value=False,
              prompt='Do you want to continue?',
              help='Assume yes to all questions')
@click.option('--dry-run', is_flag=True,
              help='Do not do anything, only show what will happen')
@click.pass_context
def sync(ctx, dry_run, data):
    # pylint: disable-msg=too-many-locals
    # pylint: disable-msg=too-many-branches
    """
    Sync Jumpcloud with a data file
    """
    if dry_run:
        LOGGER.setLevel(logging.DEBUG)

    LOGGER.debug("--- sync groups ----")
    groups = jccli_helpers.get_groups_from_file(data)
    sync_groups(ctx, groups)
    LOGGER.debug("--- sync users ----")
    users = jccli_helpers.get_users_from_file(data)
    sync_users(ctx, users)


def sync_groups(ctx, groups):
    # pylint: disable-msg=too-many-locals
    # pylint: disable-msg=too-many-branches
    """
    Sync Jumpcloud groups with definition in data file
    :param ctx: Click context
    :param groups: groups from data file
    :return:
    """
    key = ctx.obj.get('key')
    dry_run = ctx.params.get('dry_run')

    local_groups = groups

    api2 = JumpcloudApiV2(key)
    jc_group_names = []
    jc_groups_request = api2.get_groups()
    if jc_groups_request:
        for jc_group in jc_groups_request:
            jc_group_names.append(jc_group['_name'])

    LOGGER.debug(f"jumpcloud groups: {jc_group_names}")

    # create new groups
    added_groups = []
    for group in local_groups:
        do_create_group = False
        try:
            group_name = group['name']
            group_type = group['type']
            if group_name not in jc_group_names:
                do_create_group = True
            else:
                LOGGER.debug(f"{group_name} group already exists")
        except KeyError as error:
            raise error

        if do_create_group:
            added_groups.append(group_name)
            new_group = {}
            new_group['name'] = group_name
            new_group['type'] = group_type
            LOGGER.info(f"create {' '.join(group_type.split('_'))}: {group_name}")
            if not dry_run:
                response = api2.create_group(group_name, group_type)

    # remove groups that do not exist in the users file
    local_group_names = []
    for group in local_groups:
        local_group_names.append(group['name'])

    removed_groups = []
    for jc_group in jc_groups_request:
        jc_group_type = jc_group['_type']
        jc_group_name = jc_group['_name']
        do_remove_group = False
        if jc_group_name not in local_group_names:
            do_remove_group = True

        if do_remove_group:
            removed_groups.append(jc_group)
            LOGGER.info(f"remove {' '.join(jc_group_type.split('_'))}: {jc_group_name}")
            if not dry_run:
                response = api2.delete_group(jc_group_name)


def sync_users(ctx, users):
    # pylint: disable-msg=too-many-locals
    # pylint: disable-msg=too-many-branches
    # pylint: disable-msg=too-many-statements
    """
    Sync Jumpcloud users with definition in data file
    :param ctx: Click context
    :param users: users from data file
    :return:
    """
    local_users = users
    key = ctx.obj.get('key')
    dry_run = ctx.params.get('dry_run')

    api1 = JumpcloudApiV1(key)
    jc_usernames = []
    jc_emails = []
    jc_users = []
    jc_users_request = api1.get_users()
    if jc_users_request:
        for jc_user in jc_users_request:
            jc_usernames.append(jc_user['_username'])
            jc_emails.append(jc_user['_email'])
            jc_users.append({'username': jc_user['_username'], 'email': jc_user['_email']})

    LOGGER.debug(f"jumpcloud users: {jc_usernames}")

    # create new users
    api2 = JumpcloudApiV2(key)
    added_users = []
    for user in local_users:
        do_create_user = False
        try:
            user_name = user['username']
            user_email = user['email']
            if (user_name not in jc_usernames) and (user_email not in jc_emails):
                do_create_user = True
            else:
                LOGGER.debug(f"{user_name} user already exists")
        except KeyError as error:
            raise error

        if do_create_user:
            added_users.append({'username': user_name, 'email': user_email})
            new_user = {}
            new_user['username'] = user_name
            new_user['email'] = user_email
            new_user['firstname'] = user['firstname']
            new_user['firstname'] = user['lastname']
            LOGGER.info(f"create user: {user_name}")
            if not dry_run:
                response = api1.create_user(new_user)
                group_id, group_type = api2.get_group("staff")
                if group_id:
                    user_id = api1.get_user_id(user_name)
                    LOGGER.info(f"bind {user_id} to group: {group_id}")
                    if not dry_run:
                        api2.bind_user_to_group(user_id, group_id)

    # remove users that do not exist in the users file
    local_usernames = []
    local_emails = []
    for user in users:
        local_usernames.append(user['username'])
        local_emails.append(user['email'])

    removed_users = []
    for jc_user in jc_users:
        do_remove_user = False
        user_name = jc_user['username']
        user_email = jc_user['email']
        if (user_name not in local_usernames) and (user_email not in local_emails):
            do_remove_user = True

        if do_remove_user:
            removed_users.append(jc_user)
            LOGGER.info(f"remove user: {user_name}")
            if not dry_run:
                response = api1.delete_user(username=user_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

A Jumpcloud command line client
"""

from .version import __version__, __release__  # noqa



# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.errors.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

Exceptions

"""

class JcCliError(Exception):
    """
    Base class for all JC CLI errors
    """

class SystemUserNotFoundError(JcCliError):
    """
    Jumpcloud system user is not found
    """

class GroupNotFoundError(JcCliError):
    """
    Jumpcloud group is not found
    """

class MissingRequiredArgumentError(JcCliError):
    """
    Required arguments are missing
    """



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.version
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This module contains project version information.
"""

__version__ = "0.0.11"  #: the working version
__release__ = "0.0.11"  #: the release version



# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.jc_api_v2.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is a utility library for the jumpcloud version 2 api

.. note::

    To learn more about the jumpcloud api 2
    `project website <https://github.com/TheJumpCloud/jcapi-python/tree/master/jcapiv2>`_.
"""
import jcapiv2
from jcapiv2.rest import ApiException

from jccli.helpers import class_to_dict
from jccli.errors import GroupNotFoundError

VALID_USER_GROUP_TYPES = ['user', jcapiv2.GroupType.USER_GROUP]
VALID_SYSTEM_GROUP_TYPES = ['system', jcapiv2.GroupType.SYSTEM_GROUP]

class JumpcloudApiV2:
    """
        Wrapper for Jumpcloud API v2
    """
    def __init__(self, api_key):
        configuration = jcapiv2.Configuration()
        configuration.api_key['x-api-key'] = api_key
        self.graph_api = jcapiv2.GraphApi(jcapiv2.ApiClient(configuration))
        self.groups_api = jcapiv2.GroupsApi(jcapiv2.ApiClient(configuration))
        self.user_groups_api = jcapiv2.UserGroupsApi(jcapiv2.ApiClient(configuration))
        self.system_groups_api = jcapiv2.SystemGroupsApi(jcapiv2.ApiClient(configuration))
        self.bulk_job_requests_api = jcapiv2.BulkJobRequestsApi(jcapiv2.ApiClient(configuration))

    def create_group(self, name, type):
        """
        Create a Jumpcloud group
        :param name: The group name
        :param type: The group type, user or system
        :return: API response
        """
        group_name = name
        group_type = type

        if group_type in VALID_SYSTEM_GROUP_TYPES:
            try:
                body = jcapiv2.SystemGroupData(name=group_name)
                api_response = \
                    self.system_groups_api.groups_system_post(content_type='application/json',
                                                              accept='application/json',
                                                              body=body,
                                                              x_org_id='')
                return api_response
            except ApiException as error:
                raise "Exception when calling SystemGroupsApi->groups_system_post: %s\n" % error
        elif group_type in VALID_USER_GROUP_TYPES:
            try:
                body = jcapiv2.UserGroupPost(name=group_name)
                api_response = \
                    self.user_groups_api.groups_user_post(content_type='application/json',
                                                          accept='application/json',
                                                          body=body,
                                                          x_org_id='')
                return api_response
            except ApiException as error:
                raise "Exception when calling UserGroupsApi->groups_user_post: %s\n" % error
        else:
            raise ValueError("group type must be system or user")

    def delete_group(self, name):
        """
        Delete a Jumpcloud group
        :param name: The group name
        :return: API response
        """
        group_name = name
        group_id, group_type = self.get_group(group_name)

        if group_id is None:
            raise GroupNotFoundError("Group {} not found".format(name))

        if group_type == "system_group":
            try:
                api_response = \
                    self.system_groups_api.groups_system_delete(group_id,
                                                                content_type='application/json',
                                                                accept='application/json',
                                                                x_org_id='')
                return api_response
            except ApiException as error:
                raise "Exception when calling UserGroupsApi->groups_user_delete: %s\n" % error
        else:
            try:
                api_response = \
                    self.user_groups_api.groups_user_delete(group_id,
                                                            content_type='application/json',
                                                            accept='application/json',
                                                            x_org_id='')
                return api_response
            except ApiException as error:
                raise "Exception when calling UserGroupsApi->groups_user_delete: %s\n" % error

    def bind_user_to_group(self, user_id, group_id):
        """
        Associates a Jumpcloud user to a Jumpcloud group
        :param user_id:
        :param group_id:
        :return:
        """
        body = jcapiv2.UserGroupMembersReq(id=user_id,
                                           op="add",
                                           type="user")
        try:
            api_response = \
                self.graph_api.graph_user_group_members_post(group_id,
                                                             content_type='application/json',
                                                             accept='application/json',
                                                             body=body,
                                                             x_org_id='')
            return api_response
        except ApiException as error:
            raise "Exception when calling GraphApi->graph_user_group_members_post: %s\n" % error

    def bind_ldap_to_user(self, ldap_id):
        """
        Associates a Jumpcloud user to a Jumpcloud LDAP
        :param ldap_id:
        :return:
        """
        ldapserver_id = ldap_id
        body = jcapiv2.GraphManagementReq()
        try:
            api_response = \
                self.graph_api.graph_ldap_server_associations_post(ldapserver_id,
                                                                   content_type='application/json',
                                                                   accept='application/json',
                                                                   body=body,
                                                                   x_org_id='')
            return api_response
        except ApiException as error:
            raise "Exception when calling \
                   GraphApi->graph_ldap_server_associations_post: %s\n" % error

    def get_group(self, group_name, limit=100, skip=0, sort='', fields='', filter=''):
        # pylint: disable-msg=too-many-locals
        # pylint: disable-msg=too-many-arguments
        """
        Get the jumpcloud group info from a Jumpcloud group name
        :param group_name: name of the JC group
        :return:  The jumpcloud group id and type, NONE group is not found
        """
        groups = self.get_groups(limit, skip, sort, fields, filter)
        for group in groups:
            if group.name == group_name:
                return group.id, group.type
        return None, None

    def get_groups(self, limit=100, skip=0, sort='', fields='', filter=''):
        # pylint: disable-msg=too-many-locals
        # pylint: disable-msg=too-many-arguments
        """
        Get all jumpcloud groups
        :param group_name: name of the JC group
        :return: A list of jumpcloud groups
        """
        try:
            # response does not provide a total so set limit to max value
            results = self.groups_api.groups_list(content_type='application/json',
                                                  accept='application/json',
                                                  fields=fields,
                                                  filter=filter,
                                                  limit=limit,
                                                  skip=skip,
                                                  sort=sort,
                                                  x_org_id='')

            groups = class_to_dict(results)

            return groups
        except ApiException as error:
            raise "Exception when calling GroupsApi->groups_list: %s\n" % error



# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.helpers.py
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This is a set of helper methods

"""

import json
import yaml

def class_to_dict(class_object):
    """
    Convert a list of jumpcloud users to a list of dicts
    """
    new_obj = []
    for item in class_object:
        new_obj.append(item)

    return new_obj

def get_users_from_file(data_file):
    """
    Get users from a data file
    :param data_file:
    :return: a list of SystemUsers
    """
    users = []
    try:
        with open(data_file, 'r') as file:
            jc_config = yaml.safe_load(file)
            users = jc_config['users']

    except (KeyError) as error:
        pass
    except Exception as error:
        raise error

    return users

def get_groups_from_file(data_file):
    """
    Get groups from a data file
    :param data_file: data file
    :return: a list of jumpcloud groups
    """
    groups = []
    try:
        with open(data_file, 'r') as file:
            jc_config = yaml.safe_load(file)
            groups = jc_config['groups']

    except (KeyError) as error:
        pass
    except Exception as error:
        raise error

    return groups

def get_user_from_term(input):
    """
    Get user from an input string
    example:
     jccli create-user \
     --json "{\"email\": \"jc.tester1@sagebase.org\", \"username\": \"jctester1\"}"
    :param user_file:
    :return: a SystemUser
    """
    user = {}
    if input != "":
        try:
            user = json.loads(input.replace("'", '"'))

        except Exception as error:
            raise error

    return user

def get_user_from_file(user_file):
    """
    Get users from a file
    :param user_file:
    :return: a list of SystemUsers
    """
    user = {}
    try:
        with open(user_file, 'r') as file:
            user = json.load(file)

    except Exception as error:
        raise error

    return user
